@@ -36,24 +36,21 @@ namespace Tomboy
 		{
 			Console.WriteLine ("Applet Created...");
 
-			manager = new NoteManager ();
+			manager = Tomboy.DefaultNoteManager;
 			tray = new TomboyTray (manager);
 			keybinder = new TomboyGConfXKeybinder (manager, tray);
 
-			// Register the manager to handle remote requests.
-			Tomboy.RegisterRemoteControl (manager);
-
 			Add (tray);
 			ShowAll ();
 
 			// Keep around so our callbacks don't get reaped.
 			menu_verbs = new BonoboUIVerb [] {
-				new BonoboUIVerb ("Props", 
-						  new ContextMenuItemCallback (ShowPreferencesVerb)),
-				new BonoboUIVerb ("Plugins", 
-						  new ContextMenuItemCallback (ShowPluginsVerb)),
-				new BonoboUIVerb ("About", 
-						  new ContextMenuItemCallback (ShowAboutVerb))
+				new BonoboUIVerb (
+					"Props", new ContextMenuItemCallback (ShowPreferencesVerb)),
+				new BonoboUIVerb (
+					"Plugins", new ContextMenuItemCallback (ShowPluginsVerb)),
+				new BonoboUIVerb (
+					"About", new ContextMenuItemCallback (ShowAboutVerb))
 			};
 
 			// FIXME: This silently fails for some unknown reason
@@ -88,7 +85,8 @@ namespace Tomboy
 		//       backgrounds correctly, since Gtk# doesn't map
 		//       GtkStyle::bg_pixmaps[].
 		[DllImport ("libtomboy")]
-		private static extern IntPtr tomboy_widget_set_bg_pixmap (IntPtr widget, IntPtr pixmap);
+		private static extern IntPtr tomboy_widget_set_bg_pixmap (IntPtr widget, 
+									  IntPtr pixmap);
 
 		// FIXME: Overriding to this crashes in the C# bindings.
 		//        Manually tweaked generated bindings to not crash.
@@ -125,7 +123,7 @@ namespace Tomboy
 		private static extern IntPtr egg_tray_icon_new (string name);
 
 		public TomboyTrayIcon ()
-			: this (new NoteManager ())
+			: this (Tomboy.DefaultNoteManager)
 		{
 		}
 
@@ -134,9 +132,6 @@ namespace Tomboy
 			this.Raw = egg_tray_icon_new (Catalog.GetString ("Tomboy Notes"));
 			this.manager = manager;
 
-			// Register the manager to handle remote requests.
-			Tomboy.RegisterRemoteControl (manager);
-
 			tray = new TomboyTray (manager);
 			tray.ButtonPressEvent += ButtonPress;
 
