@@ -13,7 +13,6 @@ namespace Tomboy
 	{
 		string filepath;
 
-		DateTime change_date;
 		bool save_needed;
 		bool is_new;
 
@@ -23,17 +22,29 @@ namespace Tomboy
 		internal int width, height;
 		internal int x, y;
 		internal int cursor_pos;
+		internal DateTime create_date;
+		internal DateTime change_date;
+		internal string version;
 
 		NoteManager manager;
 		NoteWindow window;
 		NoteBuffer buffer;
+		NoteTagTable tag_table;
 
 		InterruptableTimeout save_timeout;
 
+		[System.Diagnostics.Conditional ("DEBUG_SAVE")]
+		static void DebugSave (string format, params object[] args)
+		{
+			Console.WriteLine (format, args);
+		}
+
 		Note ()
 		{
 			save_timeout = new InterruptableTimeout ();
 			save_timeout.Timeout += SaveTimeout;
+			x = -1;
+			y = -1;
 		}
 
 		// Create a new note stored in a file...
@@ -44,7 +55,8 @@ namespace Tomboy
 			this.filepath = filepath;
 			this.manager = manager;
 			this.is_new = true;
-			this.change_date = DateTime.Now;
+			this.create_date = DateTime.Now;
+			this.change_date = create_date;
 		}
 
 		// Internal constructor, used when loading from an exising filepath.
@@ -54,6 +66,7 @@ namespace Tomboy
 			this.manager = manager;
 			this.filepath = filepath;
 			this.is_new = false;
+			this.create_date = DateTime.MinValue;
 			this.change_date = File.GetLastWriteTime (filepath);
 		}
 
@@ -72,6 +85,14 @@ namespace Tomboy
 		{
 			Note note = new Note (read_file, manager);
 			NoteArchiver.Read (read_file, note);
+
+			if (note.version != NoteArchiver.CURRENT_VERSION) {
+				// Note has old format, so rewrite it.  No need
+				// to reread, since we are not adding anything.
+				Console.WriteLine ("Updating note XML to newest format...");
+				NoteArchiver.Write (read_file, note);
+			}
+
 			return note;
 		}
 
@@ -94,23 +115,24 @@ namespace Tomboy
 
 		void BufferChanged (object sender, EventArgs args)
 		{
+			DebugSave ("BufferChanged queueing save");
 			QueueSave (true);
 		}
 
 		void BufferTagApplied (object sender, Gtk.TagAppliedArgs args)
 		{
-			if (NoteTagTable.TagIsIgnored (args.Tag))
-				return;
-
-			QueueSave (true);
+			if (NoteTagTable.TagIsSerializable (args.Tag)) {
+				DebugSave ("BufferTagApplied queueing save: {0}", args.Tag.Name);
+				QueueSave (true);
+			}
 		}
 
 		void BufferTagRemoved (object sender, Gtk.TagRemovedArgs args)
 		{
-			if (NoteTagTable.TagIsIgnored (args.Tag))
-				return;
-
-			QueueSave (true);
+			if (NoteTagTable.TagIsSerializable (args.Tag)) {
+				DebugSave ("BufferTagRemoved queueing save: {0}", args.Tag.Name);
+				QueueSave (true);
+			}
 		}
 
 		void BufferInsertMarkSet (object sender, Gtk.MarkSetArgs args)
@@ -120,6 +142,7 @@ namespace Tomboy
 
 			cursor_pos = args.Location.Offset;
 
+			DebugSave ("BufferInsertSetMark queueing save");
 			QueueSave (false);
 		}
 
@@ -152,6 +175,7 @@ namespace Tomboy
 			width = cur_width;
 			height = cur_height;
 			
+			DebugSave ("WindowConfigureEvent queueing save");
 			QueueSave (false);
 		}
 
@@ -161,10 +185,13 @@ namespace Tomboy
 			window = null;
 		}
 
-		// Set a 4 second timeout to execute the save.  Possibly invalid the text, which
-		// causes a re-serialize when the timeout is called...
-		void QueueSave (bool invalidate_text)
+		// Set a 4 second timeout to execute the save.  Possibly
+		// invalidate the text, which causes a re-serialize when the
+		// timeout is called...
+		public void QueueSave (bool invalidate_text)
 		{
+			DebugSave ("Got QueueSave with invalidate = {0}", invalidate_text);
+
 			// Replace the existing save timeout.  Wait 4 seconds
 			// before saving...
 			save_timeout.Reset (4000);
@@ -225,7 +252,7 @@ namespace Tomboy
 		{
 			get {
 				if (text == null && buffer != null) {
-					Console.WriteLine ("Re-serializing buffer...");
+					DebugSave ("Re-serializing to XML...");
 					text = NoteBufferArchiver.Serialize (buffer); 
 				}
 				return text;
@@ -241,6 +268,8 @@ namespace Tomboy
 				}
 
 				text = value;
+
+				DebugSave ("Set of XmlContent queueing save");
 				QueueSave (false);
 			}
 		}
@@ -257,6 +286,11 @@ namespace Tomboy
 			}
 		}
 
+		public DateTime CreateDate 
+		{
+			get { return create_date; }
+		}
+
 		public DateTime ChangeDate 
 		{
 			get { return change_date; }
@@ -268,25 +302,35 @@ namespace Tomboy
 			set { manager = value; }
 		}
 
-		public NoteBuffer Buffer
+		public NoteTagTable TagTable
 		{
 			get {
-				if (buffer == null) {
-					Console.WriteLine ("Creating Buffer for '{0}'...", title);
-
+				if (tag_table == null) {
 #if FIXED_GTKSPELL
 					// NOTE: Sharing the same TagTable means
 					// that formatting is duplicated between
 					// buffers.
-					buffer = new NoteBuffer (NoteTagTable.Instance);
+					tag_table = NoteTagTable.Instance;
 #else
 					// NOTE: GtkSpell chokes on shared
 					// TagTables because it blindly tries to
 					// create a new "gtkspell-misspelling"
 					// tag, which fails if one already
 					// exists in the table.
-					buffer = new NoteBuffer (new NoteTagTable ());
+					tag_table = new NoteTagTable ();
 #endif
+				}
+				return tag_table;
+			}
+		}
+
+		public NoteBuffer Buffer
+		{
+			get {
+				if (buffer == null) {
+					Console.WriteLine ("Creating Buffer for '{0}'...", title);
+
+					buffer = new NoteBuffer (TagTable);
 
 					// Don't create Undo actions during load
 					buffer.Undoer.FreezeUndo ();
@@ -297,12 +341,15 @@ namespace Tomboy
 									text);
 					buffer.Modified = false;
 
-					// Move cursor to last-saved position
+					
 					Gtk.TextIter cursor;
-					if (cursor_pos != 0)
+					if (cursor_pos != 0) {
+						// Move cursor to last-saved position
 						cursor = buffer.GetIterAtOffset (cursor_pos);
-					else
-						cursor = buffer.GetIterAtLine (2); // avoid title line
+					} else {
+						// Avoid title line
+						cursor = buffer.GetIterAtLine (2);
+					}
 					buffer.PlaceCursor (cursor);
 
 					// New events should create Undo actions
@@ -329,13 +376,10 @@ namespace Tomboy
 					if (width != 0 && height != 0)
 						window.SetDefaultSize (width, height);
 
-					// Center new notes on screen
-					if (x == 0 && y == 0)
-						window.SetPosition (Gtk.WindowPosition.Center);
-					else
+					if (x != -1 && y != -1)
 						window.Move (x, y);
 
-					// This is here because emiting
+					// This is here because emiting inside
 					// OnRealized causes segfaults.
 					if (Opened != null)
 						Opened (this, new EventArgs ());
@@ -370,9 +414,12 @@ namespace Tomboy
 
 	public class NoteArchiver
 	{
+		public const string CURRENT_VERSION = "0.2";
+
 		public static void Read (string read_file, Note note) 
 		{
-			StreamReader reader = new StreamReader (read_file, System.Text.Encoding.UTF8);
+			StreamReader reader = new StreamReader (read_file, 
+								System.Text.Encoding.UTF8);
 			XmlTextReader xml = new XmlTextReader (reader);
 			xml.Namespaces = false;
 
@@ -381,6 +428,7 @@ namespace Tomboy
 				case XmlNodeType.Element:
 					switch (xml.Name) {
 					case "note":
+						note.version = xml.GetAttribute ("version");
 						break;
 					case "title":
 						note.title = xml.ReadString ();
@@ -390,6 +438,14 @@ namespace Tomboy
 						// NOTE: Use .text here to avoid triggering a save.
 						note.text = xml.ReadInnerXml ();
 						break;
+					case "last-change-date":
+						note.change_date = 
+							XmlConvert.ToDateTime (xml.ReadString ());
+						break;
+					case "create-date":
+						note.create_date = 
+							XmlConvert.ToDateTime (xml.ReadString ());
+						break;
 					case "cursor-position":
 						note.cursor_pos = int.Parse (xml.ReadString ());
 						break;
@@ -422,18 +478,41 @@ namespace Tomboy
 
 			xml.WriteStartDocument ();
 			xml.WriteStartElement (null, "note", "http://beatniksoftware.com/tomboy");
+			xml.WriteAttributeString(null, 
+						 "version", 
+						 null, 
+						 CURRENT_VERSION);
+			xml.WriteAttributeString("xmlns", 
+						 "link", 
+						 null, 
+						 "http://beatniksoftware.com/tomboy/link");
+			xml.WriteAttributeString("xmlns", 
+						 "size", 
+						 null, 
+						 "http://beatniksoftware.com/tomboy/size");
 
 			xml.WriteStartElement (null, "title", null);
 			xml.WriteString (note.title);
 			xml.WriteEndElement ();
 
 			xml.WriteStartElement (null, "text", null);
+			xml.WriteAttributeString ("xml", "space", null, "preserve");
 			// Insert <note-content> blob...
 			// NOTE: Use .XmlContent here to force a reget of text
 			//       from the buffer, if needed.
 			xml.WriteRaw (note.XmlContent);
 			xml.WriteEndElement ();
 
+			xml.WriteStartElement (null, "last-change-date", null);
+			xml.WriteString (XmlConvert.ToString (note.change_date));
+			xml.WriteEndElement ();
+
+			if (note.create_date != DateTime.MinValue) {
+				xml.WriteStartElement (null, "create-date", null);
+				xml.WriteString (XmlConvert.ToString (note.create_date));
+				xml.WriteEndElement ();
+			}
+
 			xml.WriteStartElement (null, "cursor-position", null);
 			xml.WriteString (note.cursor_pos.ToString ());
 			xml.WriteEndElement ();
@@ -458,18 +537,26 @@ namespace Tomboy
 			xml.WriteEndDocument ();
 			xml.Close ();
 
-			// Backup the to a ~ file, just in case...
 			if (File.Exists (write_file)) {
 				string backup_path = write_file + "~";
-
 				if (File.Exists (backup_path))
 					File.Delete (backup_path);
 
+				// Backup the to a ~ file, just in case
 				File.Move (write_file, backup_path);
+
+				// Move the temp file to write_file
+				File.Move (tmp_file, write_file);
+
+				// Delete the ~ file
+				File.Delete (backup_path);
+			} else {
+				// Move the temp file to write_file
+				File.Move (tmp_file, write_file);
 			}
 
-			// Move the temp file to write_file
-			File.Move (tmp_file, write_file);
+			// This is always the latest after a write
+			note.version = CURRENT_VERSION;
 		}
 	}
 }
