@@ -377,7 +377,7 @@ namespace Tomboy
 		// Add Undo, Redo, Link, Link To menu, Font menu to the start of
 		// the editor's context menu.
 		//
- 
+		[GLib.ConnectBefore]
 		void OnPopulatePopup (object sender, Gtk.PopulatePopupArgs args)
 		{
 			args.Menu.AccelGroup = accel_group;
@@ -507,11 +507,11 @@ namespace Tomboy
 			ToolMenuButton plugin_button = 
 				new ToolMenuButton (toolbar, 
 						    Gtk.Stock.Execute,
-						    Catalog.GetString ("_Plugins"),
+						    Catalog.GetString ("T_ools"),
 						    plugin_menu);
 			plugin_button.Show ();
 			toolbar.AppendWidget (plugin_button, 
-					      Catalog.GetString ("Run Plugin actions"), 
+					      Catalog.GetString ("Use tools on this note"), 
 					      null);
 
 			toolbar.AppendSpace ();
@@ -665,14 +665,34 @@ namespace Tomboy
 		void LinkButtonClicked () 
 		{
 			string select = note.Buffer.Selection;
+			if (select == null)
+				return;
+			
+			string body_unused;
+			string title = NoteManager.SplitTitleFromContent (select, out body_unused);
+			if (title == null)
+				return;
 
-			if (select != null) {
-				Note match = note.Manager.Find (select);
-				if (match == null)
+			Note match = note.Manager.Find (title);
+			if (match == null) {
+				try {
 					match = note.Manager.Create (select);
-
-				match.Window.Present ();
+				} catch (Exception e) {
+					HIGMessageDialog dialog = 
+						new HIGMessageDialog (
+							this,
+							Gtk.DialogFlags.DestroyWithParent,
+							Gtk.MessageType.Error,
+							Gtk.ButtonsType.Ok,
+							Catalog.GetString ("Cannot create note"),
+							e.Message);
+					dialog.Run ();
+					dialog.Destroy ();
+					return;
+				}
 			}
+
+			match.Window.Present ();
 		}
 
 		// 
