@@ -57,13 +57,80 @@ namespace Tomboy
 		{
 			Gtk.Widget parent = (Gtk.Widget) sender;
 
-			if (args.Event.Button == 1) {
+			switch (args.Event.Button) {
+			case 1:
 				Gtk.Menu recent_menu = MakeRecentNotesMenu (parent);
 				GuiUtils.PopupMenu (recent_menu, args.Event);
 				args.RetVal = true;
+				break;
+			case 2:
+				// Give some visual feedback
+				Gtk.Drag.Highlight (this);
+				args.RetVal = PastePrimaryClipboard ();
+				Gtk.Drag.Unhighlight (this);
+				break;
 			}
 		}
 
+		void PrependTimestampedText (Note note, DateTime timestamp, string text)
+		{
+			NoteBuffer buffer = note.Buffer;
+			StringBuilder insert_text = new StringBuilder ();
+
+			insert_text.Append ("\n"); // initial newline
+			string date_format = Catalog.GetString ("dddd, MMMM d, h:mm tt");
+			insert_text.Append (timestamp.ToString (date_format));
+			insert_text.Append ("\n"); // begin content
+			insert_text.Append (text);
+			insert_text.Append ("\n"); // trailing newline
+
+			buffer.Undoer.FreezeUndo ();
+
+			// Insert the date and list of links...
+			Gtk.TextIter cursor = buffer.StartIter;
+			cursor.ForwardLines (1); // skip title
+
+			buffer.Insert (cursor, insert_text.ToString ());
+
+			// Make the date string a small font...
+			cursor = buffer.StartIter;
+			cursor.ForwardLines (2); // skip title & leading newline
+
+			Gtk.TextIter end = cursor;
+			end.ForwardToLineEnd (); // end of date
+
+			buffer.ApplyTag ("datetime", cursor, end);
+
+			// Select the text we've inserted (avoid trailing newline)...
+			end = cursor;
+			end.ForwardChars (insert_text.Length - 1);
+
+			buffer.MoveMark (buffer.SelectionBound, cursor);
+			buffer.MoveMark (buffer.InsertMark, end);
+
+			buffer.Undoer.ThawUndo ();
+		}
+
+		bool PastePrimaryClipboard ()
+		{
+			Gtk.Clipboard clip = GetClipboard (Gdk.Selection.Primary);
+			string text = clip.WaitForText ();
+
+			if (text == null || text.Trim() == string.Empty)
+				return false;
+
+			Note link_note = manager.Find (Catalog.GetString ("Start Here"));
+			if (link_note == null)
+				return false;
+
+			link_note.Window.Present ();
+			PrependTimestampedText (link_note, 
+						DateTime.Now, 
+						text);
+
+			return true;
+		}
+
 		Gtk.Menu MakeRecentNotesMenu (Gtk.Widget parent) 
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
@@ -97,16 +164,15 @@ namespace Tomboy
 				if (note.IsSpecial)
 					continue;
 
-				if (note.IsOpened || 
+				if ((note.IsOpened && note.Window.IsMapped) || 
 				    note.ChangeDate > two_days_ago ||
 				    list_size < min_size) {
 					item = MakeNoteMenuItem (note);
 					menu.Append (item);
-				}
 
-				list_size++;
-				if (list_size == max_size)
-					break;
+					if (++list_size == max_size)
+					    break;
+				}
 			}
 
 			Note start = manager.Find (Catalog.GetString ("Start Here"));
@@ -122,7 +188,7 @@ namespace Tomboy
 
 			menu.Append (new Gtk.SeparatorMenuItem ());
 
-			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Recent Changes"));
+			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Table of Contents"));
 			item.Image = new Gtk.Image (Gtk.Stock.SortAscending, Gtk.IconSize.Menu);
 			item.Activated += ViewRecentChanges;
 			menu.Append (item);
@@ -178,9 +244,21 @@ namespace Tomboy
 
 		void CreateNewNote (object sender, EventArgs args) 
 		{
-			Note new_note = manager.Create ();
-			if (new_note != null)
+			try {
+				Note new_note = manager.Create ();
 				new_note.Window.Show ();
+			} catch (Exception e) {
+				HIGMessageDialog dialog = 
+					new HIGMessageDialog (
+						null,
+						0,
+						Gtk.MessageType.Error,
+						Gtk.ButtonsType.Ok,
+						Catalog.GetString ("Cannot create new note"),
+						e.Message);
+				dialog.Run ();
+				dialog.Destroy ();
+			}
 		}
 
 		void SearchNotes (object sender, EventArgs args) 
@@ -233,7 +311,7 @@ namespace Tomboy
 				new Gnome.About (
 					"Tomboy", 
 					Defines.VERSION,
-					Catalog.GetString ("Copyright \xa9 2004, 2005 Alex Graveley"),
+					Catalog.GetString ("Copyright \xa9 2004-2006 Alex Graveley"),
 					Catalog.GetString ("A simple and easy to use desktop " +
 							   "note-taking application."),
 					authors, 
@@ -275,53 +353,28 @@ namespace Tomboy
 			if (uri_list.Count == 0)
 				return;
 
-			Note link_note = manager.Find (Catalog.GetString ("Start Here"));
-			link_note.Window.Present ();
-
-			NoteBuffer buffer = link_note.Buffer;
 			StringBuilder insert_text = new StringBuilder ();
-
-			insert_text.Append ("\n"); // initial newline
-
-			string date_format = Catalog.GetString ("dddd, MMMM d, h:mm tt");
-			insert_text.Append (DateTime.Now.ToString (date_format));
+			bool more_than_one = false;
 
 			foreach (Uri uri in uri_list) {
-				insert_text.Append ("\n");
+				if (more_than_one)
+					insert_text.Append ("\n");
 
 				if (uri.IsFile) 
 					insert_text.Append (uri.LocalPath);
 				else
 					insert_text.Append (uri.ToString ());
+
+				more_than_one = true;
 			}
 
-			insert_text.Append ("\n"); // trailing newline
-
-			buffer.Undoer.FreezeUndo ();
-
-			// Insert the date and list of links...
-			Gtk.TextIter cursor = buffer.StartIter;
-			cursor.ForwardLines (1); // skip title
-
-			buffer.Insert (cursor, insert_text.ToString ());
-
-			// Make the date string a small font...
-			cursor = buffer.StartIter;
-			cursor.ForwardLines (2); // skip title & leading newline
-
-			Gtk.TextIter end = cursor;
-			end.ForwardToLineEnd (); // end of date
-
-			buffer.ApplyTag ("size:small", cursor, end);
-
-			// Select the text we've inserted (avoid trailing newline)...
-			end = cursor;
-			end.ForwardChars (insert_text.Length - 1);
-
-			buffer.MoveMark (buffer.SelectionBound, cursor);
-			buffer.MoveMark (buffer.InsertMark, end);
-
-			buffer.Undoer.ThawUndo ();
+			Note link_note = manager.Find (Catalog.GetString ("Start Here"));
+			if (link_note != null) {
+				link_note.Window.Present ();
+				PrependTimestampedText (link_note, 
+							DateTime.Now, 
+							insert_text.ToString ());
+			}
 		}
 	}
 
