@@ -11,10 +11,11 @@ namespace Tomboy
 	public class NoteRenameWatcher : NotePlugin
 	{
 		bool editing_title;
+		Gtk.TextTag title_tag;
 
 		protected override void Initialize ()
 		{
-			// Do nothing.
+			title_tag = Note.TagTable.Lookup ("note-title");
 		}
 
 		protected override void Shutdown ()
@@ -22,11 +23,6 @@ namespace Tomboy
 			// Do nothing.
 		}
 
-		Gtk.TextTag TitleTag
-		{
-			get { return Buffer.TagTable.Lookup ("note-title"); }
-		}
-
 		Gtk.TextIter TitleEnd 
 		{
 			get {
@@ -55,30 +51,30 @@ namespace Tomboy
 
 			// Clean up title line
 			Buffer.RemoveAllTags (TitleStart, TitleEnd);
-			Buffer.ApplyTag (TitleTag, TitleStart, TitleEnd);
+			Buffer.ApplyTag (title_tag, TitleStart, TitleEnd);
 		}
 
 		// This only gets called on an explicit move, not when typing
 		void OnMarkSet (object sender, Gtk.MarkSetArgs args)
 		{
-			Update ();
+			if (args.Mark == Buffer.InsertMark) {
+				Update ();
+			}
 		}
 
 		void OnInsertText (object sender, Gtk.InsertTextArgs args)
 		{
-			Changed ();
 			Update ();
 
 			Gtk.TextIter end = args.Pos;
 			end.ForwardToLineEnd ();
 
 			// Avoid lingering note-title after a multi-line insert...
-			Buffer.RemoveTag (TitleTag, TitleEnd, end);
+			Buffer.RemoveTag (title_tag, TitleEnd, end);
 		}
 
 		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
 		{
-			Changed ();
 			Update ();
 		}
 
@@ -87,11 +83,17 @@ namespace Tomboy
 			Gtk.TextIter insert = Buffer.GetIterAtMark (Buffer.InsertMark);
 			Gtk.TextIter selection = Buffer.GetIterAtMark (Buffer.SelectionBound);
 
+			// FIXME: Handle middle-click paste when insert or
+			// selection isn't on line 0, which means we won't know
+			// about the edit.
+
 			if (insert.Line == 0 || selection.Line == 0) {
 				if (!editing_title)
 					editing_title = true;
+				Changed ();
 			} else {
 				if (editing_title) {
+					Changed ();
 					UpdateNoteTitle ();
 					editing_title = false;
 				}
@@ -100,12 +102,9 @@ namespace Tomboy
 
 		void Changed ()
 		{
-			if (!editing_title)
-				return;
-
 			// Make sure the title line is big and red...
 			Buffer.RemoveAllTags (TitleStart, TitleEnd);
-			Buffer.ApplyTag (TitleTag, TitleStart, TitleEnd);
+			Buffer.ApplyTag (title_tag, TitleStart, TitleEnd);
 
 			// NOTE: Use "(Untitled #)" for empty first lines...
 			string title = TitleStart.GetText (TitleEnd).Trim ();
@@ -162,6 +161,10 @@ namespace Tomboy
 
 		void ShowNameClashError (string title)
 		{
+			// Select text from TitleStart to TitleEnd
+			Buffer.MoveMark (Buffer.SelectionBound, TitleStart);
+			Buffer.MoveMark (Buffer.InsertMark, TitleEnd);
+
 			string message = 
 				String.Format (Catalog.GetString ("A note with the title " +
 								  "<b>{0}</b> already exists. " +
@@ -241,7 +244,7 @@ namespace Tomboy
 		void FixupOldGtkSpell ()
 		{
 #if OLD_GTKSPELL
-			Gtk.TextTag misspell = Buffer.TagTable.Lookup ("gtkspell-misspelled");
+			Gtk.TextTag misspell = Note.TagTable.Lookup ("gtkspell-misspelled");
 			if (misspell != null) {
 				OldGtkSpellFixup hack = new OldGtkSpellFixup (misspell.Handle);
 				hack.Fixup ();
@@ -281,35 +284,32 @@ namespace Tomboy
 
 		void TagApplied (object sender, Gtk.TagAppliedArgs args)
 		{
+			bool remove = false;
+
 			if (args.Tag.Name == "gtkspell-misspelled") {
 				// Remove misspelled tag for links & title
 				foreach (Gtk.TextTag tag in args.StartChar.Tags) {
-					if (tag.Name != null && 
-					    (tag.Name.StartsWith ("link:") || 
-					     tag.Name == "note-title")) {
-						Buffer.RemoveTag ("gtkspell-misspelled", 
-								  args.StartChar, 
-								  args.EndChar);
+					if (tag != args.Tag && 
+					    !NoteTagTable.TagIsSpellCheckable (tag)) {
+						remove = true;
 						break;
 					}
 				}
-			} else if (args.Tag.Name != null && 
-				   (args.Tag.Name.StartsWith ("link:") ||
-				    args.Tag.Name == "note-title")) {
-				Gtk.TextTag misspell = 
-					Buffer.TagTable.Lookup ("gtkspell-misspelled");
-				if (misspell != null) {
-					Buffer.RemoveTag ("gtkspell-misspelled", 
-							  args.StartChar, 
-							  args.EndChar);
-				}
+			} else if (!NoteTagTable.TagIsSpellCheckable (args.Tag)) {
+				remove = true;
+			}
+
+			if (remove) {
+				Buffer.RemoveTag ("gtkspell-misspelled", 
+						  args.StartChar, 
+						  args.EndChar);
 			}
 		}
 	}
 
 	public class NoteUrlWatcher : NotePlugin
 	{
-		Gtk.TextTag url_tag;
+		NoteTag url_tag;
 		Gtk.TextMark click_mark;
 
 		const string URL_REGEX = 
@@ -327,7 +327,7 @@ namespace Tomboy
 
 		protected override void Initialize ()
 		{
-			// Do nothing.
+			url_tag = (NoteTag) Note.TagTable.Lookup ("link:url");
 		}
 
 		protected override void Shutdown ()
@@ -337,12 +337,6 @@ namespace Tomboy
 
 		protected override void OnNoteOpened ()
 		{
-			url_tag = Buffer.TagTable.Lookup ("link:url");
-			if (url_tag == null) {
-				Console.WriteLine ("Tag 'link:url' not registered for buffer");
-				return;
-			}
-
 #if FIXED_GTKSPELL
 			// NOTE: This hack helps avoid multiple URL opens for
 			// cases where the GtkSpell version is fixed to allow
@@ -352,11 +346,11 @@ namespace Tomboy
 			// multiple times for each button press.  Fixes bug
 			// #305813.
 			if (!text_event_connected) {
-				url_tag.TextEvent += OnTextEvent;
+				url_tag.Activated += OnUrlTagActivated;
 				text_event_connected = true;
 			}
 #else
-			url_tag.TextEvent += OnTextEvent;
+			url_tag.Activated += OnUrlTagActivated;
 #endif
 
 			click_mark = Buffer.CreateMark (null, Buffer.StartIter, true);
@@ -369,14 +363,8 @@ namespace Tomboy
 			Window.Editor.PopupMenu += OnPopupMenu;
 		}
 
-		string GetUrlAtIter (Gtk.TextIter iter)
+		string GetUrl (Gtk.TextIter start, Gtk.TextIter end)
 		{
-			Gtk.TextIter start = iter;
-			start.BackwardToTagToggle (url_tag);
-
-			Gtk.TextIter end = iter;
-			end.ForwardToTagToggle (url_tag);
-
 			string url = start.GetText (end);
 
 			// FIXME: Needed because the file match is greedy and
@@ -388,11 +376,6 @@ namespace Tomboy
 			// to /home/alex/foo.
 			if (url.StartsWith ("www."))
 				url = "http://" + url;
-			else if (url.StartsWith ("http://") ||
-				 url.StartsWith ("https://") ||
-				 url.StartsWith ("ftp://") ||
-				 url.StartsWith ("file://"))
-				url = url;
 			else if (url.StartsWith ("/") && 
 				 url.LastIndexOf ("/") > 1)
 				url = "file://" + url;
@@ -431,42 +414,28 @@ namespace Tomboy
 			dialog.Destroy ();
 		}
 
-		void OnTextEvent (object sender, Gtk.TextEventArgs args)
+		bool OnUrlTagActivated (NoteTag      sender,
+					NoteEditor   editor,
+					Gtk.TextIter start, 
+					Gtk.TextIter end)
 		{
-			Gtk.TextTag tag = (Gtk.TextTag) sender;
-
-			if (args.Event.Type != Gdk.EventType.ButtonPress)
-				return;
-
-			Gdk.EventButton button_ev = new Gdk.EventButton (args.Event.Handle);
-			if (button_ev.Button != 1 && button_ev.Button != 2)
-				return;
-
-			/* Don't open link if Shift or Control is pressed */
-			if ((int) (button_ev.State & (Gdk.ModifierType.ShiftMask |
-						      Gdk.ModifierType.ControlMask)) != 0)
-				return;
-
-			string url = GetUrlAtIter (args.Iter);
+			string url = GetUrl (start, end);
 			try {
 				OpenUrl (url);
-
-				// Close note on middle-click
-				if (button_ev.Button == 2) {
-					Window.Hide ();
-				}
 			} catch (GLib.GException e) {
 				ShowOpeningLocationError (url, e.Message);
 			}
 
 			// Kill the middle button paste...
-			args.RetVal = true;
+			return true;
 		}
 
 		void ApplyUrlToBlock (Gtk.TextIter start, Gtk.TextIter end)
 		{
-			start.LineOffset = 0;
-			end.ForwardToLineEnd ();
+			NoteBuffer.GetBlockExtents (ref start, 
+						    ref end, 
+						    256 /* max url length */,
+						    url_tag);
 
 			Buffer.RemoveTag (url_tag, start, end);
 
@@ -557,18 +526,21 @@ namespace Tomboy
 		void OpenLinkActivate (object sender, EventArgs args)
 		{
 			Gtk.TextIter click_iter = Buffer.GetIterAtMark (click_mark);
-			string url = GetUrlAtIter (click_iter);
-			try {
-				OpenUrl (url);
-			} catch (GLib.GException e) {
-				ShowOpeningLocationError (url, e.Message);
-			}
+
+			Gtk.TextIter start, end;
+			url_tag.GetExtents (click_iter, out start, out end);
+
+			OnUrlTagActivated (url_tag, (NoteEditor) Window.Editor, start, end);
 		}
 
 		void CopyLinkActivate (object sender, EventArgs args)
 		{
 			Gtk.TextIter click_iter = Buffer.GetIterAtMark (click_mark);
-			string url = GetUrlAtIter (click_iter);
+
+			Gtk.TextIter start, end;
+			url_tag.GetExtents (click_iter, out start, out end);
+
+			string url = GetUrl (start, end);
 
 			Gtk.Clipboard clip = Window.Editor.GetClipboard (Gdk.Selection.Clipboard);
 			clip.SetText (url);
@@ -577,11 +549,21 @@ namespace Tomboy
 
 	public class NoteLinkWatcher : NotePlugin
 	{
+		NoteTag url_tag;
+		NoteTag link_tag;
+		NoteTag broken_link_tag;
+
+		static bool text_event_connected;
+
 		protected override void Initialize () 
 		{
 			Manager.NoteDeleted += OnNoteDeleted;
 			Manager.NoteAdded += OnNoteAdded;
 			Manager.NoteRenamed += OnNoteRenamed;
+
+			url_tag = (NoteTag) Note.TagTable.Lookup ("link:url");
+			link_tag = (NoteTag) Note.TagTable.Lookup ("link:internal");
+			broken_link_tag = (NoteTag) Note.TagTable.Lookup ("link:broken");
 		}
 
 		protected override void Shutdown ()
@@ -593,6 +575,23 @@ namespace Tomboy
 
 		protected override void OnNoteOpened ()
 		{
+#if FIXED_GTKSPELL
+			// NOTE: This avoid multiple link opens for cases where
+			// the GtkSpell version is fixed to allow TagTable
+			// sharing.  This is because if the TagTable is shared,
+			// we will connect to the same Tag's event source each
+			// time a note is opened, and get called multiple times
+			// for each button press.  Fixes bug #305813.
+			if (!text_event_connected) {
+				link_tag.Activated += OnLinkTagActivated;
+				broken_link_tag.Activated += OnLinkTagActivated;
+				text_event_connected = true;
+			}
+#else
+			link_tag.Activated += OnLinkTagActivated;
+			broken_link_tag.Activated += OnLinkTagActivated;
+#endif
+
 			Buffer.InsertText += OnInsertText;
 			Buffer.DeleteRange += OnDeleteRange;
 		}
@@ -614,7 +613,7 @@ namespace Tomboy
 				return;
 
 			// Highlight previously unlinked text
-			Highlighter.HighlightNote (Note, Buffer.StartIter, Buffer.EndIter, added);
+			HighlightInBlock (Buffer.StartIter, Buffer.EndIter);
 		}
 
 		void OnNoteDeleted (object sender, Note deleted)
@@ -625,9 +624,6 @@ namespace Tomboy
 			if (!ContainsText (deleted.Title))
 				return;
 
-			Gtk.TextTag link_tag = Buffer.TagTable.Lookup ("link:internal");
-			Gtk.TextTag broken_link_tag = Buffer.TagTable.Lookup ("link:broken");
-
 			string old_title_lower = deleted.Title.ToLower ();
 
 			// Turn all link:internal to link:broken for the deleted note.
@@ -647,17 +643,12 @@ namespace Tomboy
 				return;
 
 			// Highlight previously unlinked text
-			if (ContainsText (renamed.Title)) {
-				Highlighter.HighlightNote (Note, 
-							   Buffer.StartIter, 
-							   Buffer.EndIter, 
-							   renamed);
-			}
+			if (ContainsText (renamed.Title))
+				HighlightNoteInBlock (renamed, Buffer.StartIter, Buffer.EndIter);
 
 			if (!ContainsText (old_title))
 				return;
 
-			Gtk.TextTag link_tag = Buffer.TagTable.Lookup ("link:internal");
 			string old_title_lower = old_title.ToLower ();
 
 			// Replace existing links with the new title.
@@ -675,111 +666,75 @@ namespace Tomboy
 			}
 		}
 
-		class Highlighter
+		void DoHighlight (TrieHit hit, Gtk.TextIter start, Gtk.TextIter end)
 		{
-			Note note;
-			Gtk.TextIter start;
-			Gtk.TextIter end;
+			Note hit_note = (Note) hit.Value;
+			if (hit_note == this.Note)
+				return;
 
-			Gtk.TextTag url_tag;
-			Gtk.TextTag link_tag;
-			Gtk.TextTag broken_link_tag;
+			Gtk.TextIter title_start = start;
+			title_start.ForwardChars (hit.Start);
 
-			Highlighter (Note         note, 
-				     Gtk.TextIter start, 
-				     Gtk.TextIter end)
-			{
-				this.note = note;
-				this.start = start;
-				this.end = end;
+			// Don't create links inside URLs
+			if (title_start.HasTag (url_tag))
+				return;
 
-				this.url_tag = note.Buffer.TagTable.Lookup ("link:url");
-				this.link_tag = note.Buffer.TagTable.Lookup ("link:internal");
-				this.broken_link_tag = note.Buffer.TagTable.Lookup ("link:broken");
-			}
+			Gtk.TextIter title_end = start;
+			title_end.ForwardChars (hit.End);
+			
+			Console.WriteLine ("Matching Note title '{0}' at {1}-{2}...", 
+					   hit.Key,
+					   hit.Start,
+					   hit.End);
 
-			public static void HighlightTrie (Note         note, 
-							  Gtk.TextIter start, 
-							  Gtk.TextIter end,
-							  TrieTree     trie)
-			{
-				Highlighter high = new Highlighter (note, start, end);
-				trie.FindMatches (start.GetText (end), 
-						  new MatchHandler (high.TitleFound));
-			}
+			Buffer.RemoveTag (broken_link_tag, title_start, title_end);
+			Buffer.ApplyTag (link_tag, title_start, title_end);
+		}
 
-			public static void HighlightNote (Note         note, 
-							  Gtk.TextIter start, 
-							  Gtk.TextIter end,
-							  Note         find_note)
-			{
-				Highlighter high = new Highlighter (note, start, end);
+		void HighlightNoteInBlock (Note find_note, Gtk.TextIter start, Gtk.TextIter end) 
+		{
+			string buffer_text = start.GetText (end).ToLower();
+			string find_title_lower = find_note.Title.ToLower ();
+			int idx = 0;
 
-				string buffer_text = start.GetText (end).ToLower();
-				string find_title_lower = find_note.Title.ToLower ();
-				int idx = 0;
+			while (true) {
+				idx = buffer_text.IndexOf (find_title_lower, idx);
+				if (idx < 0)
+					break;
 
-				while (true) {
-					idx = buffer_text.IndexOf (find_title_lower, idx);
-					if (idx < 0)
-						break;
+				TrieHit hit = new TrieHit (idx, 
+							   idx + find_title_lower.Length,
+							   find_title_lower,
+							   find_note);
+				DoHighlight (hit, start, end);
 
-					high.TitleFound (buffer_text, idx, find_note);
-					idx += find_title_lower.Length;
-				}
-			}
-
-			void TitleFound (string haystack, int start_idx, object value)
-			{
-				Note match_note = (Note) value;
-
-				if (match_note == note)
-					return;
-
-				Gtk.TextIter title_start = start;
-				title_start.ForwardChars (start_idx);
-
-				// Don't create links inside URLs
-				if (title_start.HasTag (url_tag))
-					return;
-
-				Gtk.TextIter title_end = title_start;
-				title_end.ForwardChars (match_note.Title.Length);
-
-				Console.WriteLine ("Matching Note title '{0}'...", 
-						   match_note.Title);
-
-				note.Buffer.RemoveTag (broken_link_tag, title_start, title_end);
-				note.Buffer.ApplyTag (link_tag, title_start, title_end);
+				idx += find_title_lower.Length;
 			}
 		}
 
 		void HighlightInBlock (Gtk.TextIter start, Gtk.TextIter end) 
 		{
-			GetBlockExtents (ref start, ref end);
-			Highlighter.HighlightTrie (Note, start, end, Manager.TitleTrie);
+			ArrayList hits = Manager.TitleTrie.FindMatches (start.GetText (end));
+			foreach (TrieHit hit in hits) {
+				DoHighlight (hit, start, end);
+			}
 		}
 
 		void UnhighlightInBlock (Gtk.TextIter start, Gtk.TextIter end) 
 		{
-			Gtk.TextTag link_tag = Buffer.TagTable.Lookup ("link:internal");
 			Buffer.RemoveTag (link_tag, start, end);
 		}
 
-		void GetBlockExtents (ref Gtk.TextIter start, ref Gtk.TextIter end) 
-		{
-			// FIXME: Should only be processing the largest title
-			// size, so we don't slow down for large paragraphs 
-
-			start.LineOffset = 0;
-			end.ForwardToLineEnd ();
-		}
-
 		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
 		{
 			Gtk.TextIter start = args.Start;
 			Gtk.TextIter end = args.End;
 
+			NoteBuffer.GetBlockExtents (ref start, 
+						    ref end, 
+						    Manager.TitleTrie.MaxLength,
+						    link_tag);
+
 			UnhighlightInBlock (start, end);
 			HighlightInBlock (start, end);
 		}
@@ -791,9 +746,45 @@ namespace Tomboy
 
 			Gtk.TextIter end = args.Pos;
 
+			NoteBuffer.GetBlockExtents (ref start, 
+						    ref end, 
+						    Manager.TitleTrie.MaxLength,
+						    link_tag);
+
 			UnhighlightInBlock (start, end);
 			HighlightInBlock (start, end);
 		}
+
+		bool OpenOrCreateLink (Gtk.TextIter start, Gtk.TextIter end)
+		{
+			string link_name = start.GetText (end);
+			Note link = Manager.Find (link_name);
+
+			if (link == null) {
+				Console.WriteLine ("Creating note '{0}'...", link_name);
+				try {
+				    link = Manager.Create (link_name);
+				} catch (Exception e) {
+				    // Fail silently.
+				}
+			}
+
+			if (link != null && link != this.Note) {
+				Console.WriteLine ("Opening note '{0}' on click...", link_name);
+				link.Window.Present ();
+				return true;
+			}
+
+			return false;
+		}
+
+		bool OnLinkTagActivated (NoteTag      sender,
+					 NoteEditor   editor,
+					 Gtk.TextIter start, 
+					 Gtk.TextIter end)
+		{
+			return OpenOrCreateLink (start, end);
+		}
 	}
 
 	public class NoteWikiWatcher : NotePlugin
@@ -812,7 +803,7 @@ namespace Tomboy
 
 		protected override void Initialize ()
 		{
-			// Do nothing.
+			broken_link_tag = Note.TagTable.Lookup ("link:broken");
 		}
 
 		protected override void Shutdown ()
@@ -822,12 +813,6 @@ namespace Tomboy
 
 		protected override void OnNoteOpened ()
 		{
-			broken_link_tag = Buffer.TagTable.Lookup ("link:broken");
-			if (broken_link_tag == null) {
-				Console.WriteLine ("ERROR: Broken link tags not registered.");
-				return;
-			}
-
 			if ((bool) Preferences.Get (Preferences.ENABLE_WIKIWORDS)) {
 				Buffer.InsertText += OnInsertText;
 				Buffer.DeleteRange += OnDeleteRange;
@@ -865,8 +850,10 @@ namespace Tomboy
 
 		void ApplyWikiwordToBlock (Gtk.TextIter start, Gtk.TextIter end)
 		{
-			start.LineOffset = 0;
-			end.ForwardToLineEnd ();
+			NoteBuffer.GetBlockExtents (ref start, 
+						    ref end, 
+						    80 /* max wiki name */,
+						    broken_link_tag);
 
 			Buffer.RemoveTag (broken_link_tag, start, end);
 
@@ -910,20 +897,15 @@ namespace Tomboy
 
 	public class MouseHandWatcher : NotePlugin
 	{
-		Gtk.TextTag link_tag;
-		Gtk.TextTag broken_link_tag;
-
 		bool hovering_on_link;
 
 		static Gdk.Cursor normal_cursor;
 		static Gdk.Cursor hand_cursor;
-		static bool text_event_connected;
 
 		static MouseHandWatcher ()
 		{
 			normal_cursor = new Gdk.Cursor (Gdk.CursorType.Xterm);
 			hand_cursor = new Gdk.Cursor (Gdk.CursorType.Hand2);
-			text_event_connected = false;
 		}
 
 		protected override void Initialize ()
@@ -938,84 +920,12 @@ namespace Tomboy
 
 		protected override void OnNoteOpened ()
 		{
-			link_tag = Buffer.TagTable.Lookup ("link:internal");
-			broken_link_tag = Buffer.TagTable.Lookup ("link:broken");
-
-#if FIXED_GTKSPELL
-			// NOTE: This avoid multiple link opens for cases where
-			// the GtkSpell version is fixed to allow TagTable
-			// sharing.  This is because if the TagTable is shared,
-			// we will connect to the same Tag's event source each
-			// time a note is opened, and get called multiple times
-			// for each button press.  Fixes bug #305813.
-			if (!text_event_connected) {
-				link_tag.TextEvent += OnLinkTextEvent;
-				broken_link_tag.TextEvent += OnLinkTextEvent;
-				text_event_connected = true;
-			}
-#else
-			link_tag.TextEvent += OnLinkTextEvent;
-			broken_link_tag.TextEvent += OnLinkTextEvent;
-#endif
-
 			Gtk.TextView editor = Window.Editor;
 			editor.MotionNotifyEvent += OnEditorMotion;
 			editor.KeyPressEvent += OnEditorKeyPress;
 			editor.KeyReleaseEvent += OnEditorKeyRelease;
 		}
 
-		bool OpenOrCreateLink (Gtk.TextIter start, Gtk.TextIter end)
-		{
-			string link_name = start.GetText (end);
-			Note link = Manager.Find (link_name);
-
-			if (link == null) {
-				Console.WriteLine ("Creating note '{0}'...", link_name);
-				link = Manager.Create (link_name);
-			}
-
-			if (link != null && link != this.Note) {
-				Console.WriteLine ("Opening note '{0}' on click...", link_name);
-				link.Window.Present ();
-				return true;
-			}
-
-			return false;
-		}
-
-		void OnLinkTextEvent (object sender, Gtk.TextEventArgs args)
-		{
-			Gtk.TextTag tag = (Gtk.TextTag) sender;
-
-			if (args.Event.Type != Gdk.EventType.ButtonPress)
-				return;
-
-			Gdk.EventButton button_ev = new Gdk.EventButton (args.Event.Handle);
-			if (button_ev.Button != 1 && button_ev.Button != 2)
-				return;
-
-			/* Don't open link if Shift or Control is pressed */
-			if ((int) (button_ev.State & (Gdk.ModifierType.ShiftMask |
-						      Gdk.ModifierType.ControlMask)) != 0)
-				return;
-
-			Gtk.TextIter start = args.Iter, end = args.Iter;
-
-			if (!start.BeginsTag (tag))
-				start.BackwardToTagToggle (tag);
-			end.ForwardToTagToggle (tag);
-
-			if (!OpenOrCreateLink (start, end))
-				return;
-
-			if (button_ev.Button == 2) {
-				Window.Hide ();
-
-				// Kill the middle button paste...
-				args.RetVal = true;
-			}
-		}
-
 		[GLib.ConnectBefore]
 		void OnEditorKeyPress (object sender, Gtk.KeyPressEventArgs args) 
 		{
@@ -1036,27 +946,15 @@ namespace Tomboy
 
 			case Gdk.Key.Return:
 			case Gdk.Key.KP_Enter:
-				// Control-Enter opens the link at point...
-
-				// FIXME: just fire a Widget.Event for this
-				// args.Event, and let the handlers deal
-
-				if ((int) (args.Event.State & Gdk.ModifierType.ControlMask) == 0)
-					break;
-				
 				Gtk.TextIter iter = Buffer.GetIterAtMark (Buffer.InsertMark);
 
 				foreach (Gtk.TextTag tag in iter.Tags) {
-					if (tag == link_tag || tag == broken_link_tag) {
-						Gtk.TextIter start = iter, end = iter;
-
-						if (!start.BeginsTag (tag))
-							start.BackwardToTagToggle (tag);
-						end.ForwardToTagToggle (tag);
-
-						OpenOrCreateLink (start, end);
-						args.RetVal = true;
-						break;
+					if (NoteTagTable.TagIsActivatable (tag)) {
+						args.RetVal = tag.ProcessEvent (Window.Editor,
+										args.Event,
+										iter);
+						if ((bool) args.RetVal)
+							break;
 					}
 				}
 				break;
@@ -1106,8 +1004,7 @@ namespace Tomboy
 			Gtk.TextIter iter = Window.Editor.GetIterAtLocation (buffer_x, buffer_y);
 
 			foreach (Gtk.TextTag tag in iter.Tags) {
-				if (tag.Name != null && 
-				    tag.Name.StartsWith ("link:")) {
+				if (NoteTagTable.TagIsActivatable (tag)) {
 					hovering = true;
 					break;
 				}
